#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_anp_epg_contract
short_description: Manage EPG contracts in schema templates
description:
- Manage EPG contracts in schema templates on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
- Akini Ross (@akinross)
options:
  schema:
    description:
    - The name of the Schema.
    type: str
    required: true
  template:
    description:
    - The name of the Template.
    type: str
    required: true
  anp:
    description:
    - The name of the ANP.
    type: str
    required: true
  epg:
    description:
    - The name of the EPG.
    type: str
    required: true
  force_replace:
    description:
    - Replaces all the configured contract(s) with the provided contract(s).
    - This option can only be used in combination with the O(contracts) option.
    - In combination with the O(state=absent) and without any contract configuration all configured static port(s) will be removed.
    type: bool
  contract:
    description:
    - The Contract associated to this EPG.
    - This option can not be used in combination with the I(contracts) option.
    type: dict
    suboptions:
      name:
        description:
        - The name of the Contract.
        required: true
        type: str
      schema:
        description:
        - The name of the Schema that defines the referenced Contract.
        - If this parameter is unspecified, it defaults to the current schema.
        type: str
      template:
        description:
        - The name of the Template that defines the referenced Contract.
        - If this parameter is unspecified, it defaults to the current schema.
        type: str
      type:
        description:
        - The type of the Contract.
        type: str
        required: true
        choices: [ consumer, provider ]
  contracts:
    description:
    - A list of Contracts associated to this EPG.
    - This option can not be used in combination with the O(contract) option.
    - All configured contract(s) will be replaced with the provided contract(s) when used with O(force_replace=true).
    - Only the provided contract(s) will be added, updated or removed when used with O(force_replace=false).
    - In combination with the O(state=query) all provided contract(s) must be found else the task will fail.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the Contract.
        required: true
        type: str
      schema:
        description:
        - The name of the Schema that defines the referenced Contract.
        - If this parameter is unspecified, it defaults to the current schema.
        type: str
      template:
        description:
        - The name of the Template that defines the referenced Contract.
        - If this parameter is unspecified, it defaults to the current schema.
        type: str
      type:
        description:
        - The type of the Contract.
        type: str
        required: true
        choices: [ consumer, provider ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_anp_epg
- module: cisco.mso.mso_schema_template_contract_filter
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a contract to an EPG
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contract:
      name: Contract 1
      type: consumer
    state: present

- name: Add 2 contracts to an EPG
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contracts:
      - name: Contract 1
        type: provider
      - name: Contract 1
        type: consumer
    state: present

- name: Replace all existing contracts on an EPG with 2 new contracts
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    force_replace: true
    contracts:
      - name: Contract 2
        type: provider
      - name: Contract 2
        type: consumer
    state: present

- name: Query a specific Contract
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contract:
      name: Contract 1
      type: consumer
    state: query
  register: query_result

- name: Query a list of Contracts
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contracts:
      - name: Contract 2
        type: provider
      - name: Contract 2
        type: consumer
    state: query
  register: query_result

- name: Query all Contracts
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    state: query
  register: query_result

- name: Remove a Contract
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contract:
      name: Contract 1
    state: absent

- name: Remove 2 contracts to an EPG
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    contracts:
      - name: Contract 1
        type: provider
      - name: Contract 1
        type: consumer
    state: absent

- name: Remove all existing contracts from an EPG
  cisco.mso.mso_schema_template_anp_epg_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    force_replace: true
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_contractref_spec
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        anp=dict(type="str", required=True),
        epg=dict(type="str", required=True),
        force_replace=dict(type="bool"),
        contract=dict(type="dict", options=mso_contractref_spec()),
        contracts=dict(type="list", elements="dict", options=mso_contractref_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["contract", "contracts", "force_replace"], True],
            ["state", "present", ["contract", "contracts"], True],
        ],
        mutually_exclusive=[
            ["contract", "contracts"],
            ["contract", "force_replace"],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    anp = module.params.get("anp")
    epg = module.params.get("epg")
    force_replace = module.params.get("force_replace")
    contract = module.params.get("contract")
    contracts = module.params.get("contracts")
    state = module.params.get("state")

    mso = MSOModule(module)

    mso_schema = MSOSchema(mso, schema, template)
    mso_schema.set_template(template)
    mso_schema.set_template_anp(anp)
    mso_schema.set_template_anp_epg(epg)

    # Schema dict is used as a cache store for schema id lookups
    # This is done to limit the amount of schema id lookups when schema is not specified for multiple contracts
    schema_cache = {mso_schema.schema_name: mso_schema.id}
    if contract:
        overwrite_contract_schema_and_template(mso, contract, schema, schema_cache, template)
    elif contracts:
        for contract_dict in contracts:
            overwrite_contract_schema_and_template(mso, contract_dict, schema, schema_cache, template)

    contracts_path = "/templates/{0}/anps/{1}/epgs/{2}/contractRelationships".format(template, anp, epg)
    contract_path = "{0}/-".format(contracts_path)
    ops = []

    if contract:
        contract_ref = mso.contract_ref(**contract)
        mso_schema.set_template_anp_epg_contract(contract_ref, contract.get("type"), False)
        if mso_schema.schema_objects.get("template_anp_epg_contract") is not None:
            mso.existing = get_contract_payload_from_schema(mso, mso_schema)
            contract_path = "{0}/{1}".format(contracts_path, mso_schema.schema_objects["template_anp_epg_contract"].index)
    else:
        found_contracts = []
        set_existing_contracts(mso, mso_schema)
        if contracts:
            for contract_details in contracts:
                contract_ref = mso.contract_ref(**contract_details)
                mso_schema.set_template_anp_epg_contract(contract_ref, contract_details.get("type"), False)
                if mso_schema.schema_objects.get("template_anp_epg_contract") is not None:
                    found_contracts.append(get_contract_payload_from_schema(mso, mso_schema))

    if state == "query":
        if contracts:
            if len(found_contracts) == len(contracts):
                mso.existing = found_contracts
            else:
                not_found_contracts = [
                    "Contract with Reference '{0}' and type '{1}' not found".format(mso.contract_ref(**contract), contract.get("type"))
                    for contract in contracts
                    if contract not in found_contracts
                ]
                mso.fail_json(msg=not_found_contracts)
        elif contract and not mso.existing:
            mso.fail_json(msg="Contract with Reference '{0}' and type '{1}' not found".format(contract_ref, contract.get("type")))
        mso.exit_json()

    mso.previous = mso.existing

    if state == "absent" and mso.existing:
        if contract:
            mso.sent = mso.proposed = {}
            ops.append(dict(op="remove", path=contract_path))
        elif force_replace:
            mso.sent = mso.proposed = []
            ops.append(dict(op="remove", path=contracts_path))
        else:
            mso.proposed = mso.existing.copy()
            remove_index = []
            for contract in contracts:
                payload = get_contract_payload(contract)
                if any(True if payload == found_contract else False for found_contract in found_contracts):
                    mso.proposed.remove(payload)
                    remove_index.append(mso.existing.index(payload))
            # The list index should not shift when removing contracts from the list
            # By sorting the indexes found in reverse order, we assure that the highest index is removed first by the NDO backend
            # This logic is to avoid removing the wrong contract
            for index in reversed(sorted(remove_index)):
                ops.append(dict(op="remove", path="{0}/{1}".format(contracts_path, index)))
            mso.sent = mso.proposed

    elif state == "present":
        if contract:
            mso.sanitize(get_contract_payload(contract), collate=True)
            if not mso.existing:
                ops.append(dict(op="add", path=contract_path, value=mso.sent))
        elif force_replace:
            mso.sent = mso.proposed = [get_contract_payload(contract) for contract in contracts]
            if mso.existing:
                ops.append(dict(op="replace", path=contracts_path, value=mso.sent))
            else:
                ops.append(dict(op="add", path=contracts_path, value=mso.sent))
        else:
            mso.sent = []
            mso.proposed = mso.existing.copy()
            for contract in contracts:
                payload = get_contract_payload(contract)
                if payload not in mso.existing:
                    mso.proposed.append(payload)
                # Only add the operation list if the contract is not already present
                # This is to avoid adding the same contract multiple times
                # Replace operation is not required because there are no attributes that can be changed except the contract itself
                if not force_replace and not any(True if payload == found_contract else False for found_contract in found_contracts):
                    ops.append(dict(op="add", path=contract_path, value=payload))
                    mso.sent.append(payload)

    mso.existing = mso.proposed

    if not module.check_mode and mso.proposed != mso.previous:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


def set_existing_contracts(mso, mso_schema):
    mso.existing = []
    for existing_contract in mso_schema.schema_objects["template_anp_epg"].details.get("contractRelationships"):
        mso.existing.append(
            dict(
                relationshipType=existing_contract.get("relationshipType"),
                contractRef=mso.dict_from_ref(existing_contract.get("contractRef")),
            )
        )


def get_contract_payload_from_schema(mso, mso_schema):
    return dict(
        relationshipType=mso_schema.schema_objects["template_anp_epg_contract"].details.get("relationshipType"),
        contractRef=mso.dict_from_ref(mso_schema.schema_objects["template_anp_epg_contract"].details.get("contractRef")),
    )


def get_contract_payload(contract):
    return dict(
        relationshipType=contract.get("type"),
        contractRef=dict(
            contractName=contract.get("name"),
            templateName=contract.get("template"),
            schemaId=contract.get("schema_id"),
        ),
    )


def overwrite_contract_schema_and_template(mso, contract, epg_schema_name, schema_cache, epg_template_name):
    if contract.get("schema") is None:
        contract["schema"] = epg_schema_name
        contract["schema_id"] = schema_cache.get(epg_schema_name)
    else:
        schema_id = schema_cache.get(contract.get("schema"))
        contract["schema_id"] = schema_id if schema_id else mso.lookup_schema(contract.get("schema"))

    if contract.get("template") is None:
        contract["template"] = epg_template_name
    else:
        contract["template"] = contract.get("template").replace(" ", "")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, DataDope (@datadope-io)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_listen_ports_facts
short_description: Recopilates the facts of the listening ports of the machine
description:
    - Recopilates the information of the TCP and UDP ports of the machine and
      the related processes.
    - State of the TCP ports could be filtered, as well as the format of the
      date when the parent process was launched.
    - The module's goal is to replicate the functionality of the linux module
      listen_ports_facts, mantaining the format of the said module.
version_added: 2.7.0
options:
  date_uformat:
    description:
      - The format of the date when the process that owns the port started.
      - The date specification is UFormat
    type: str
    default: '%c'
  tcp_filter:
    description:
      - Filter for the state of the TCP ports that will be recopilated.
      - Supports multiple states (Bound, Closed, CloseWait, Closing, DeleteTCB,
        Established, FinWait1, FinWait2, LastAck, Listen, SynReceived, SynSent
        and TimeWait), that can be used alone or combined. Note that the Bound
        state is only available on PowerShell version 4.0 or later.
    type: list
    elements: str
    default: [ Listen ]
notes:
- The generated data (tcp_listen and udp_listen) and the fields within follows
  the listen_ports_facts schema to achieve compatibility with the said module
  output, even though this module if capable of extracting ports with a state
  other than Listen
author:
- David Nieto (@david-ns)
'''

EXAMPLES = r'''
- name: Recopilate ports facts
  ansible.windows.win_listen_ports_facts:

- name: Retrieve only ports with Closing and Established states
  ansible.windows.win_listen_ports_facts:
    tcp_filter:
      - Closing
      - Established

- name: Get ports facts with only the year within the date field
  ansible.windows.win_listen_ports_facts:
    date_uformat: '%Y'
'''

RETURN = r'''
tcp_listen:
    description: List of dicts with the detected TCP ports
    returned: success
    type: list
    elements: dict
    sample: [
        {
            "address": "127.0.0.1",
            "name": "python",
            "pid": 5332,
            "port": 82,
            "protocol": "tcp",
            "stime": "Thu Nov 18 15:27:42 2021",
            "user": "SERVER\\Administrator"
        }
    ]
udp_listen:
    description: List of dicts with the detected UDP ports
    returned: success
    type: list
    elements: dict
    sample: [
        {
            "address": "127.0.0.1",
            "name": "python",
            "pid": 5332,
            "port": 82,
            "protocol": "udp",
            "stime": "Thu Nov 18 15:27:42 2021",
            "user": "SERVER\\Administrator"
        }
    ]
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_alerts_settings
short_description: Resource module for networks _alerts _settings
description:
  - Manage operation update of the resource networks _alerts _settings.
  - Update the alert configuration for this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  alerts:
    description: Alert-specific configuration for each type. Only alerts that pertain
      to the network can be updated.
    elements: dict
    suboptions:
      alertDestinations:
        description: A hash of destinations for this specific alert.
        suboptions:
          allAdmins:
            description: If true, then all network admins will receive emails for
              this alert.
            type: bool
          emails:
            description: A list of emails that will receive information about the
              alert.
            elements: str
            type: list
          httpServerIds:
            description: A list of HTTP server IDs to send a Webhook to for this alert.
            elements: str
            type: list
          smsNumbers:
            description: A list of phone numbers that will receive text messages about
              the alert. Only available for sensors status alerts.
            elements: str
            type: list
          snmp:
            description: If true, then an SNMP trap will be sent for this alert if
              there is an SNMP trap server configured for this network.
            type: bool
        type: dict
      enabled:
        description: A boolean depicting if the alert is turned on or off.
        type: bool
      filters:
        description: A hash of specific configuration data for the alert. Only filters
          specific to the alert will be updated.
        suboptions:
          conditions:
            description: Conditions.
            elements: dict
            suboptions:
              direction:
                description: Direction.
                type: str
              duration:
                description: Duration.
                type: int
              threshold:
                description: Threshold.
                type: float
              type:
                description: Type of condition.
                type: str
              unit:
                description: Unit.
                type: str
            type: list
          failureType:
            description: Failure Type.
            type: str
          lookbackWindow:
            description: Loopback Window (in sec).
            type: int
          minDuration:
            description: Min Duration.
            type: int
          name:
            description: Name.
            type: str
          period:
            description: Period.
            type: int
          priority:
            description: Priority.
            type: str
          regex:
            description: Regex.
            type: str
          selector:
            description: Selector.
            type: str
          serials:
            description: Serials.
            elements: str
            type: list
          ssidNum:
            description: SSID Number.
            type: int
          tag:
            description: Tag.
            type: str
          threshold:
            description: Threshold.
            type: int
          timeout:
            description: Timeout.
            type: int
        type: dict
      type:
        description: The type of alert.
        type: str
    type: list
  defaultDestinations:
    description: The network-wide destinations for all alerts on the network.
    suboptions:
      allAdmins:
        description: If true, then all network admins will receive emails.
        type: bool
      emails:
        description: A list of emails that will receive the alert(s).
        elements: str
        type: list
      httpServerIds:
        description: A list of HTTP server IDs to send a Webhook to.
        elements: str
        type: list
      snmp:
        description: If true, then an SNMP trap will be sent if there is an SNMP trap
          server configured for this network.
        type: bool
    type: dict
  muting:
    description: Mute alerts under certain conditions.
    suboptions:
      byPortSchedules:
        description: Mute wireless unreachable alerts based on switch port schedules.
        suboptions:
          enabled:
            description: If true, then wireless unreachable alerts will be muted when
              caused by a port schedule.
            type: bool
        type: dict
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks updateNetworkAlertsSettings
    description: Complete reference of the updateNetworkAlertsSettings API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-alerts-settings
notes:
  - SDK Method used are
    networks.Networks.update_network_alerts_settings,
  - Paths used are
    put /networks/{networkId}/alerts/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_alerts_settings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    alerts:
      - alertDestinations:
          allAdmins: false
          emails:
            - miles@meraki.com
          httpServerIds:
            - aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
          smsNumbers:
            - '+15555555555'
          snmp: false
        enabled: true
        filters:
          conditions:
            - direction: +
              duration: 0
              threshold: 72.5
              type: temperature
              unit: celsius
          failureType: 802.1X auth fail
          lookbackWindow: 360
          minDuration: 60
          name: Filter
          period: 1800
          priority: ''
          regex: '[a-z]'
          selector: '{"smartSensitivity":"medium","smartEnabled":false,"eventReminderPeriodSecs":10800}'
          serials:
            - Q234-ABCD-0001
            - Q234-ABCD-0002
            - Q234-ABCD-0003
          ssidNum: 1
          tag: tag1
          threshold: 30
          timeout: 60
        type: gatewayDown
    defaultDestinations:
      allAdmins: true
      emails:
        - miles@meraki.com
      httpServerIds:
        - aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
      snmp: true
    muting:
      byPortSchedules:
        enabled: true
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "defaultDestinations": {
        "emails": [
          "string"
        ],
        "allAdmins": true,
        "snmp": true,
        "httpServerIds": [
          "string"
        ]
      },
      "alerts": [
        {
          "type": "string",
          "enabled": true,
          "alertDestinations": {
            "emails": [
              "string"
            ],
            "smsNumbers": [
              "string"
            ],
            "allAdmins": true,
            "snmp": true,
            "httpServerIds": [
              "string"
            ]
          },
          "filters": {
            "conditions": [
              {
                "type": "string",
                "unit": "string",
                "duration": 0,
                "direction": "string",
                "threshold": 0
              }
            ],
            "failureType": "string",
            "lookbackWindow": 0,
            "minDuration": 0,
            "name": "string",
            "period": 0,
            "priority": "string",
            "regex": "string",
            "selector": "string",
            "serials": [
              "string"
            ],
            "ssidNum": 0,
            "tag": "string",
            "threshold": 0,
            "timeout": 0
          }
        }
      ],
      "muting": {
        "byPortSchedules": {
          "enabled": true
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Gaspard Micol (@gmicol) <gmicol@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_endpoint_mac_tag_policy
version_added: "2.13.0"
short_description: Manage Endpoint MAC Tag Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Endpoint MAC Tag Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v4.1 (NDO v5.1) and later.
author:
- Gaspard Micol (@gmicol)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Tenant Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Tenant Policy template.
    - This parameter or O(template) is required.
    type: str
  endpoint_mac_address:
    description:
    - The endpoint MAC address of the Endpoint MAC Tag Policy.
    type: str
    aliases: [ mac ]
  uuid:
    description:
    - The UUID of the Endpoint MAC Tag Policy.
    - The UUID must be used when updating O(endpoint_mac_address), O(bridge_domain) or O(vrf).
    type: str
    aliases: [ endpoint_mac_tag_policy_uuid ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant Policy template.
- The O(bridge_domain) must exist before using this module in your playbook.
  Use M(cisco.mso.mso_schema_template_bd) to create the Bridge Domain.
- The O(vrf) must exist before using this module in your playbook.
  Use M(cisco.mso.mso_schema_template_vrf) to create the VRF.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.mso_schema_template_bd
- module: cisco.mso.mso_schema_template_vrf
extends_documentation_fragment:
- cisco.mso.modules
- cisco.mso.bridge_domain_references
- cisco.mso.vrf_references
- cisco.mso.annotations
- cisco.mso.policy_tags
"""

EXAMPLES = r"""
- name: Create an Endpoint MAC Tag Policy
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 01:23:45:67:89:AB
    bridge_domain:
      reference:
        name: ansible_bd
        template: ansible_tenant_template
        schema: ansible_schema
    annotations:
      - key: key_1
        value: value_1
      - key: key_2
        value: value_2
    policy_tags:
      - key: key_1
        value: value_1
      - key: key_2
        value: value_2
    state: present
  register: create_endpoint_mac_tag_policy

- name: Update an Endpoint MAC Tag Policy using MAC address and Bridge Domain
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 01:23:45:67:89:AB
    bridge_domain:
      reference:
        name: ansible_bd
        template: ansible_tenant_template
        schema: ansible_schema
    annotations:
      - key: key_1
        value: value_1
    policy_tags:
      - key: key_1
        value: value_1
      - key: key_2
        value: value_2
      - key: key_3
        value: value_3
    state: present

- name: Update an Endpoint MAC Tag Policy using UUID
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    uuid: "{{ create_endpoint_mac_tag_policy.current.uuid }}"
    endpoint_mac_address: 2C:54:91:88:C9:E3
    vrf:
      reference:
        name: ansible_vrf
        template: ansible_tenant_template
        schema: ansible_schema
    annotations:
      - key: key_1
        value: value_1
    policy_tags:
      - key: key_1
        value: value_1
      - key: key_2
        value: value_2
      - key: key_3
        value: value_3
    state: present

- name: Query an Endpoint MAC Tag Policy using MAC address and Bridge Domain
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 01:23:45:67:89:AB
    bridge_domain:
      reference:
        name: ansible_bd
        template: ansible_tenant_template
        schema: ansible_schema
    state: query
  register: query_with_mac_and_bd

- name: Query an Endpoint MAC Tag Policy using MAC address and VRF
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 2C:54:91:88:C9:E3
    vrf:
      reference:
        name: ansible_vrf
        template: ansible_tenant_template
        schema: ansible_schema
    state: query
  register: query_with_mac_and_vrf

- name: Query an Endpoint MAC Tag Policy using UUID
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_endpoint_mac_tag_policy.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all Endpoint MAC Tag Policies in a template
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    state: query
  register: query_all_objects

- name: Delete an Endpoint MAC Tag Policy using its MAC address and Bridge Domain
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 01:23:45:67:89:AB
    bridge_domain:
      reference:
        name: ansible_bd
        template: ansible_tenant_template
        schema: ansible_schema
    state: absent

- name: Delete an Endpoint MAC Tag Policy using its MAC address and VRF
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    endpoint_mac_address: 01:23:45:67:89:AB
    vrf:
      reference:
        name: ansible_vrf
        template: ansible_tenant_template
        schema: ansible_schema
    state: absent

- name: Delete an Endpoint MAC Tag Policy using UUID
  cisco.mso.ndo_endpoint_mac_tag_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy_template
    uuid: "{{ create_endpoint_mac_tag_policy.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
    ndo_schema_template_object_references_spec,
    ndo_tags_annotations_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, format_annotations_list
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        endpoint_mac_address=dict(type="str", aliases=["mac"]),
        uuid=dict(type="str", aliases=["endpoint_mac_tag_policy_uuid"]),
        bridge_domain=ndo_schema_template_object_references_spec(aliases=["bd"]),
        vrf=ndo_schema_template_object_references_spec(),
        annotations=ndo_tags_annotations_spec(),
        policy_tags=ndo_tags_annotations_spec(aliases=["tags"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
            ("bridge_domain", "vrf"),
        ],
        required_if=[
            ["state", "absent", ["endpoint_mac_address", "uuid"], True],
            ["state", "present", ["endpoint_mac_address", "uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    mac = module.params.get("endpoint_mac_address")
    uuid = module.params.get("uuid")
    bd = module.params.get("bridge_domain")
    vrf = module.params.get("vrf")
    annotations = module.params.get("annotations")
    policy_tags = module.params.get("policy_tags")
    state = module.params.get("state")
    reference_dict = {
        "bd": {
            "name": "bdName",
            "reference": "bdRef",
            "type": "bd",
            "template": "bdTemplateName",
            "templateId": "bdTemplateId",
            "schema": "bdSchemaName",
            "schemaId": "bdSchemaId",
        },
        "vrf": {
            "name": "vrfName",
            "reference": "vrfRef",
            "type": "vrf",
            "template": "vrfTemplateName",
            "templateId": "vrfTemplateId",
            "schema": "vrfSchemaName",
            "schemaId": "vrfSchemaId",
        },
    }

    if mac and not uuid and not (bd or vrf):
        mso.fail_json(msg="when providing a MAC address without UUID, one of the following is required: bridge_domain, vrf")

    mso_template = mso_templates.get_template("tenant", template_name, template_id)
    mso_template.validate_template("tenantPolicy")

    tenant_id = mso_template.template.get("tenantPolicyTemplate", {}).get("template", {}).get("tenantId")
    templates_objects_path = "templates/objects"
    vrf_uuid = bd_uuid = None
    if vrf:
        vrf_uuid = vrf.get("uuid")
        if not vrf_uuid:
            vrf_object = mso_template.get_vrf_object(vrf.get("reference"), tenant_id, templates_objects_path)
            vrf_uuid = vrf_object.details.get("uuid")
    elif bd:
        bd_uuid = bd.get("uuid")
        if not bd_uuid:
            bd_object = mso_template.get_bd_object(bd.get("reference"), tenant_id, templates_objects_path)
            bd_uuid = bd_object.details.get("uuid")

    match = mso_template.get_endpoint_mac_tag_policy_object(uuid, mac, bd_uuid, vrf_uuid)

    if (uuid or (mac and (bd_uuid or vrf_uuid))) and match:
        mso.existing = mso.previous = copy.deepcopy(
            mso_template.update_config_with_template_and_references(match.details, reference_dict)
        )  # Query a specific object
    elif match:
        mso.existing = [mso_template.update_config_with_template_and_references(obj, reference_dict) for obj in match]  # Query all objects

    endpoint_mac_tag_policy_path = "/tenantPolicyTemplate/template/endpointMacTagPolicies/{0}".format(match.index if match else "-")

    ops = []

    if state == "present":

        mso_values = {
            "mac": mac,
            "bdRef": bd_uuid,
            "vrfRef": vrf_uuid,
        }

        format_annotations_list(mso_values, annotations)
        if policy_tags:
            mso_values["policyTags"] = policy_tags

        if match:
            remove_data = []
            if annotations == []:
                remove_data.append("tagAnnotations")
            if policy_tags == []:
                remove_data.append("policyTags")
            if bd_uuid and match.details.get("vrfRef"):
                remove_data.append("vrfRef")
            elif match.details.get("bdRef") and vrf_uuid:
                remove_data.append("bdRef")
            append_update_ops_data(ops, match.details, endpoint_mac_tag_policy_path, mso_values, remove_data)
            mso.sanitize(mso_values, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=endpoint_mac_tag_policy_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=endpoint_mac_tag_policy_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = mso_template.get_endpoint_mac_tag_policy_object(uuid, mac, bd_uuid, vrf_uuid, search_object=response)
        if match:
            mso.existing = mso_template.update_config_with_template_and_references(match.details, reference_dict)  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        if mso.proposed:
            mso_template.update_config_with_template_and_references(mso.proposed, reference_dict)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_overview_by_model_info
short_description: Information module for organizations _devices _overview _by _model
description:
  - Get all organizations _devices _overview _by _model.
  - Lists the count for each device model.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  models:
    description:
      - >
        Models query parameter. Optional parameter to filter devices by one or more
        models. All returned devices will have a model that is an exact match.
    elements: str
    type: list
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter devices by networkId.
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. Optional parameter to filter device by device
        product types. This filter uses multiple exact matches.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesOverviewByModel
    description: Complete reference of the getOrganizationDevicesOverviewByModel API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-overview-by-model
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_overview_by_model,
  - Paths used are
    get /organizations/{organizationId}/devices/overview/byModel,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _overview _by _model
  cisco.meraki.organizations_devices_overview_by_model_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    models: []
    networkIds: []
    productTypes: []
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "model": "string",
        "total": 0
      }
    ]
"""



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.routesoperations?view=azure-python
#
from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_afdroute
version_added: "3.4.0"
short_description: Manage an Azure Front Door Route
description:
    - Create, update and delete an Azure Front Door Route to be used by a Front Door Service Profile created using azure_rm_cdnprofile.

options:
    content_types_to_compress:
        description:
            - The caching configuration/compression settings for this route.
            - List of content types (str) on which compression applies. The value should be a valid MIME type.
            - Required together (is_compression_enabled, content_types_to_compress, query_string_caching_behavior, query_parameters)
        type: list
        elements: str
    custom_domains:
        description:
            - Domain names referenced by this endpoint. ID will be looked up based on the name.
        type: list
        elements: str
    disable_cache_configuration:
        description:
            - To disable cache configuration, set this to true and do not include cache_configuration.
            - This will override any cache_configuration settings you include.
        type: bool
        default: false
    enabled_state:
        description:
            - Whether to enable use of this rule. Permitted values are 'Enabled' or 'Disabled'.
        type: str
        choices:
            - Enabled
            - Disabled
    endpoint_name:
        description:
            - Name of the endpoint under the profile which is unique globally.
        required: true
        type: str
    forwarding_protocol:
        description:
            - Protocol this rule will use when forwarding traffic to backends.
        type: str
        choices:
            - HttpOnly
            - HttpsOnly
            - MatchRequest
    https_redirect:
        description:
            - Whether to automatically redirect HTTP traffic to HTTPS traffic.
        type: str
        default: Disabled
        choices:
            - Enabled
            - Disabled
    is_compression_enabled:
        description:
            - The caching configuration/compression settings for this route.
            - Indicates whether content compression is enabled on AzureFrontDoor.
            - If compression is enabled, content will be served as compressed if user requests for a compressed version.
            - Content won't be compressed on AzureFrontDoor when requested content is smaller than 1 byte or larger than 1 MB.
            - Required together (is_compression_enabled, content_types_to_compress, query_string_caching_behavior, query_parameters)
        type: bool
    link_to_default_domain:
        description:
            - whether this route will be linked to the default endpoint domain.
        type: str
        default: Disabled
        choices:
            - Enabled
            - Disabled
    name:
        description:
            - Name of the routing rule.
        required: true
        type: str
    origin_group:
        description:
            - A reference to the origin group.
        type: str
    origin_path:
        description:
            - A directory path on the origin that AzureFrontDoor can use to retrieve content from, e.g. contoso.cloudapp.net/originpath.
        type: str
    patterns_to_match:
        description:
            - The route patterns of the rule.
        type: list
        elements: str
    profile_name:
        description:
            - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.
        required: true
        type: str
    query_parameters:
        description:
            - The caching configuration for this route.
            - query parameters to include or exclude (comma separated).
            - Required together (is_compression_enabled, content_types_to_compress, query_string_caching_behavior, query_parameters)
        type: str
    query_string_caching_behavior:
        description:
            - The caching configuration for this route.
            - Defines how Frontdoor caches requests that include query strings.
            - You can ignore any query strings when caching, ignore specific query strings,
            - cache every request with a unique URL, or cache specific query strings.
            - Required together (is_compression_enabled, content_types_to_compress, query_string_caching_behavior, query_parameters)
        type: str
        choices:
            - IgnoreQueryString
            - IgnoreSpecifiedQueryStrings
            - IncludeSpecifiedQueryStrings
            - UseQueryString
    resource_group:
        description:
            - Name of the Resource group within the Azure subscription.
        required: true
        type: str
    rule_sets:
        description:
            - List of rule set names referenced by this endpoint.
        type: list
        elements: str
    state:
        description:
            - Assert the state of the Route. Use C(present) to create or update a CDN profile and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
    supported_protocols:
        description:
            - List of supported protocols for this route.
        type: list
        elements: str
        default: ['Http', 'Https']
        choices:
            - Http
            - Https

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Create an AFD Route
  azure_rm_afdroute:
    name: myRoute
    endpoint_name: myEndpoint
    origin_group: myOriginGroup
    profile_name: myProfile
    resource_group_name: myResourceGroup
    state: present
    route:
      enabled_state: Disabled
      forwarding_protocol: HttpsOnly
      https_redirect: Enabled
      patterns_to_match:
        - "/*"
      rule_sets:
        - Security
      supported_protocols:
        - Https
        - Http
      link_to_default_domain: Enabled

- name: Delete an AFD Origin
  azure_rm_afdroute:
    name: myRoute
    endpoint_name: myEndpoint
    origin_group: myOriginGroup
    profile_name: myProfile
    resource_group_name: myResourceGroup
    state: absent
'''
RETURN = '''
id:
    description:
        - ID of the Route.
    returned: always
    type: str
    sample: "id: '/subscriptions/xxxxxx-xxxx-xxxx-xxxx-xxxxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/afdendpoints/myEP/routes/myRoute'"
'''
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
# from azure.core.serialization import NULL as AzureCoreNull

try:
    from azure.mgmt.cdn.models import Route, RouteUpdateParameters, CompressionSettings, \
        ResourceReference, AfdRouteCacheConfiguration
except ImportError as ec:
    # This is handled in azure_rm_common
    pass


class AzureRMRoute(AzureRMModuleBase):
    ''' Main Class '''
    def __init__(self):
        self.module_arg_spec = dict(
            content_types_to_compress=dict(
                type='list',
                elements='str'
            ),
            custom_domains=dict(
                type='list',
                elements='str'
            ),
            disable_cache_configuration=dict(
                type='bool',
                default=False
            ),
            enabled_state=dict(
                type='str',
                choices=['Enabled', 'Disabled']
            ),
            endpoint_name=dict(
                type='str',
                required=True
            ),
            forwarding_protocol=dict(
                type='str',
                choices=['HttpOnly', 'HttpsOnly', 'MatchRequest']
            ),
            https_redirect=dict(
                type='str',
                choices=['Enabled', 'Disabled'],
                default='Disabled'
            ),
            is_compression_enabled=dict(
                type='bool'
            ),
            link_to_default_domain=dict(
                type='str',
                choices=['Enabled', 'Disabled'],
                default='Disabled'
            ),
            name=dict(
                type='str',
                required=True
            ),
            origin_group=dict(
                type='str'
            ),
            origin_path=dict(
                type='str'
            ),
            patterns_to_match=dict(
                type='list',
                elements='str'
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            query_string_caching_behavior=dict(
                type='str',
                choices=['IgnoreQueryString', 'IgnoreSpecifiedQueryStrings', 'IncludeSpecifiedQueryStrings', 'UseQueryString']
            ),
            query_parameters=dict(
                type='str'
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            rule_sets=dict(
                type='list',
                elements='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            supported_protocols=dict(
                type='list',
                elements='str',
                choices=['Http', 'Https'],
                default=['Http', 'Https']
            )
        )

        self.content_types_to_compress = None
        self.custom_domains = None
        self.disable_cache_configuration = None
        self.enabled_state = None
        self.forwarding_protocol = None
        self.https_redirect = None
        self.is_compression_enabled = None
        self.link_to_default_domain = None
        self.origin_path = None
        self.origin_group = None
        self.patterns_to_match = None
        self.query_parameters = None
        self.query_string_caching_behavior = None
        self.rule_sets = []
        self.supported_protocols = None

        self.origin_group_id = None

        self.name = None
        self.endpoint_name = None
        self.profile_name = None
        self.resource_group = None
        self.state = None

        self.rule_set_ids = []
        self.custom_domain_ids = []

        required_together = [['is_compression_enabled', 'content_types_to_compress', 'query_string_caching_behavior', 'query_parameters']]

        self.results = dict(changed=False)

        super(AzureRMRoute, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False,
            required_together=required_together
        )

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        to_be_updated = False

        # Ignore query_parameters when query_string_caching_behavior is "IgnoreQueryString" or "UseQueryString"
        if self.query_string_caching_behavior in ["IgnoreQueryString", "UseQueryString"]:
            self.query_parameters = None
            self.log("Ignoring query_parameters when query_string_caching_behavior is IgnoreQueryString or UseQueryString")

        # Get the existing resource
        response = self.get_route()

        if self.state == 'present':
            # Get the Origin Group ID
            self.origin_group_id = self.get_origin_group_id()
            if self.origin_group_id is False:
                self.fail("Could not obtain Origin Group ID from {0}, please create it first.".format(self.origin_group))

            # Get a list of all the Custom Domain IDs
            if isinstance(self.custom_domains, list):
                if len(self.custom_domains) > 0:
                    for custom_domain in self.custom_domains:
                        cd_id = self.get_custom_domain_id(custom_domain)
                        if cd_id:
                            ref = ResourceReference(
                                id=cd_id)
                            self.custom_domain_ids.append(ref)

            # Populate the rule_set_ids
            convert_rules = self.get_rule_set_ids()
            if not convert_rules:
                self.fail("Failed to convert the Rule Set names to IDs")

            if not response:
                self.log("Need to create the Route")

                if not self.check_mode:
                    new_results = self.create_route()
                    self.results['id'] = new_results['id']
                self.results['changed'] = True

            else:
                self.log('Results : {0}'.format(response))

                to_be_updated = self.update_needed(response)

                self.results['id'] = response['id']
                if to_be_updated:
                    self.log("Need to update the Route")

                    if not self.check_mode:
                        new_results = self.update_route()
                        self.results['id'] = new_results['id']

                    self.results['changed'] = True

        elif self.state == 'absent':
            if not response:
                self.log("Route {0} does not exist.".format(self.name))
                self.results['id'] = None
                self.results['changed'] = False
            else:
                self.log("Need to delete the Route")
                self.results['changed'] = True

                if not self.check_mode:
                    self.delete_route()
                    self.results['id'] = response['id']
        return self.results

    def update_needed(self, response):
        '''
        Check if the resource needs to be updated

        return: bool
        '''
        to_be_updated = False
        cache_configuration = response['cache_configuration']
        if cache_configuration and self.disable_cache_configuration:
            to_be_updated = True
        if cache_configuration:
            if cache_configuration.query_parameters != self.query_parameters and self.query_parameters:
                to_be_updated = True
            if cache_configuration.query_string_caching_behavior != self.query_string_caching_behavior and self.query_string_caching_behavior:
                to_be_updated = True
            if cache_configuration.compression_settings.is_compression_enabled != self.is_compression_enabled and self.is_compression_enabled:
                to_be_updated = True
            if cache_configuration.compression_settings.content_types_to_compress != self.content_types_to_compress and self.content_types_to_compress:
                to_be_updated = True
        else:
            if self.is_compression_enabled or self.query_parameters or self.query_string_caching_behavior or self.content_types_to_compress:
                to_be_updated = True
        # Test for custom_domain equality
        equal = True
        if len(response['custom_domains']) != len(self.custom_domain_ids):
            equal = False
        for x in response['custom_domains']:
            found = False
            for y in self.custom_domain_ids:
                if x.id == y.id:
                    found = True
                    continue
            if not found:
                equal = False
        if not equal:
            to_be_updated = True

        if response['enabled_state'] != self.enabled_state and self.enabled_state:
            to_be_updated = True
        if response['forwarding_protocol'] != self.forwarding_protocol and self.forwarding_protocol:
            to_be_updated = True
        if response['https_redirect'] != self.https_redirect and self.https_redirect:
            to_be_updated = True
        if response['link_to_default_domain'] != self.link_to_default_domain and self.link_to_default_domain:
            to_be_updated = True
        if response['origin_group_id'] != self.origin_group_id and self.origin_group_id:
            to_be_updated = True
        if response['origin_path'] != self.origin_path and self.origin_path:
            to_be_updated = True
        if response['patterns_to_match'] != self.patterns_to_match and self.patterns_to_match:
            to_be_updated = True
        if response["rule_sets"] != self.rule_set_ids and self.rule_set_ids:
            to_be_updated = True
        if response['supported_protocols'] != self.supported_protocols and self.supported_protocols:
            to_be_updated = True

        return to_be_updated

    def create_route(self):
        '''
        Creates a Azure Route.

        :return: deserialized Azure Route instance state dictionary
        '''
        self.log("Creating the Azure Route instance {0}".format(self.name))
        cache_configuration = None
        compression_settings = None

        if self.disable_cache_configuration:
            # cache_configuration = AzureCoreNull # Reported as issue to azure-mgmt-cdn: https://github.com/Azure/azure-sdk-for-python/issues/35801
            cache_configuration = None
        else:
            if not self.is_compression_enabled and not self.content_types_to_compress:
                compression_settings = None
            else:
                compression_settings = CompressionSettings(
                    content_types_to_compress=self.content_types_to_compress,
                    is_compression_enabled=self.is_compression_enabled
                )
            if not self.query_string_caching_behavior and not self.query_parameters and not compression_settings:
                cache_configuration = None
            else:
                cache_configuration = AfdRouteCacheConfiguration(
                    query_string_caching_behavior=self.query_string_caching_behavior,
                    query_parameters=self.query_parameters,
                    compression_settings=compression_settings
                )

        origin_group = ResourceReference(
            id=self.origin_group_id
        )

        parameters = Route(
            cache_configuration=cache_configuration,
            custom_domains=self.custom_domain_ids,
            enabled_state=self.enabled_state,
            forwarding_protocol=self.forwarding_protocol,
            https_redirect=self.https_redirect,
            link_to_default_domain=self.link_to_default_domain,
            origin_group=origin_group,
            origin_path=self.origin_path,
            patterns_to_match=self.patterns_to_match,
            rule_sets=self.rule_set_ids,
            supported_protocols=self.supported_protocols
        )

        try:
            poller = self.cdn_client.routes.begin_create(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name,
                route_name=self.name,
                route=parameters
            )
            response = self.get_poller_result(poller)
            return route_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create Azure Route instance.')
            self.fail("Error Creating Azure Route instance: {0}".format(str(exc)))

    def update_route(self):
        '''
        Updates an Azure Route.

        :return: deserialized Azure Route instance state dictionary
        '''
        self.log("Updating the Azure Route instance {0}".format(self.name))

        cache_configuration = None
        compression_settings = None

        if self.disable_cache_configuration:
            # cache_configuration = AzureCoreNull # Reported as issue to azure-mgmt-cdn: https://github.com/Azure/azure-sdk-for-python/issues/35801
            cache_configuration = None
        else:
            if not self.is_compression_enabled and not self.content_types_to_compress:
                compression_settings = None
            else:
                compression_settings = CompressionSettings(
                    content_types_to_compress=self.content_types_to_compress,
                    is_compression_enabled=self.is_compression_enabled
                )
            if not self.query_string_caching_behavior and not self.query_parameters and not compression_settings:
                cache_configuration = None
            else:
                cache_configuration = AfdRouteCacheConfiguration(
                    query_string_caching_behavior=self.query_string_caching_behavior,
                    query_parameters=self.query_parameters,
                    compression_settings=compression_settings
                )

        origin_group = ResourceReference(
            id=self.origin_group_id
        )

        parameters = RouteUpdateParameters(
            cache_configuration=cache_configuration,
            custom_domains=self.custom_domain_ids,
            enabled_state=self.enabled_state,
            forwarding_protocol=self.forwarding_protocol,
            https_redirect=self.https_redirect,
            link_to_default_domain=self.link_to_default_domain,
            origin_group=origin_group,
            origin_path=self.origin_path,
            patterns_to_match=self.patterns_to_match,
            rule_sets=self.rule_set_ids,
            supported_protocols=self.supported_protocols
        )

        try:
            poller = self.cdn_client.routes.begin_update(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name,
                route_name=self.name,
                route_update_properties=parameters
            )
            response = self.get_poller_result(poller)
            return route_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to update Azure Route instance.')
            self.fail("Error updating Azure Route instance: {0}".format(str(exc)))

    def delete_route(self):
        '''
        Deletes the specified Azure Route in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Route {0}".format(self.name))
        try:
            poller = self.cdn_client.routes.begin_delete(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name,
                route_name=self.name
            )
            self.get_poller_result(poller)
            return True
        except Exception as exc:
            self.log('Error attempting to delete the Route.')
            self.fail("Error deleting the Route: {0}".format(str(exc)))
            return False

    def get_route(self):
        '''
        Gets the properties of the specified Route.

        :return: deserialized Route state dictionary
        '''
        self.log(
            "Checking if the Route {0} is present".format(self.name))
        try:
            response = self.cdn_client.routes.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name,
                route_name=self.name,
            )
            self.log("Response : {0}".format(response))
            self.log("Route : {0} found".format(response.name))
            return route_to_dict(response)
        except Exception as err:
            self.log('Did not find the Route.' + err.args[0])
            return False

    def get_origin_group_id(self):
        '''
        Gets the ID of the specified Origin Group.

        :return: ID for the Origin Group.
        '''
        self.log(
            "Obtaining ID for Origin Group {0}".format(self.origin_group))
        try:
            response = self.cdn_client.afd_origin_groups.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                origin_group_name=self.origin_group
            )
            self.log("Response : {0}".format(response))
            self.log("Origin Group ID found : {0} found".format(response.id))
            return response.id
        except Exception as err:
            self.log('Did not find the Origin Group.' + err.args[0])
            return False

    def get_custom_domain_id(self, custom_domain):
        '''
        Gets the ID of the specified Custom Domain.

        :return: ID for the Custom Domain.
        '''
        self.log("Obtaining ID for Custom Domain {0}".format(self.origin_group))
        try:
            response = self.cdn_client.afd_custom_domains.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                custom_domain_name=custom_domain
            )
            self.log("Response : {0}".format(response))
            self.log("Custom Domain found : {0} found".format(response.id))
            return response.id
        except Exception as err:
            self.log('Did not find the Custom Domain.' + err.args[0])
            return False

    def get_rule_set_ids(self):
        '''
        Gets the IDs of the specified Rule Sets.

        :return: Boolean if Rule Sets were found and translated.
        '''
        if self.rule_sets is None or len(self.rule_sets) == 0:
            return True

        self.log("Obtaining IDs for Rule Sets")

        try:
            for rule_name in self.rule_sets:
                response = self.cdn_client.rule_sets.get(
                    resource_group_name=self.resource_group,
                    profile_name=self.profile_name,
                    rule_set_name=rule_name,
                )
                self.log("Response : {0}".format(response))
                self.log("Rule Set ID found : {0} found".format(response.id))
                self.rule_set_ids.append(ResourceReference(id=response.id))
            return True
        except Exception as err:
            self.log('Error getting the Rule Set IDs.' + err.args[0])
            return False


def route_to_dict(route):
    '''
        Convert the object to dictionary
    '''
    return dict(
        custom_domains=route.custom_domains,
        cache_configuration=route.cache_configuration,
        deployment_status=route.deployment_status,
        enabled_state=route.enabled_state,
        forwarding_protocol=route.forwarding_protocol,
        https_redirect=route.https_redirect,
        id=route.id,
        link_to_default_domain=route.link_to_default_domain,
        name=route.name,
        origin_group_id=route.origin_group.id,
        origin_path=route.origin_path,
        patterns_to_match=route.patterns_to_match,
        provisioning_state=route.provisioning_state,
        rule_sets=route.rule_sets,
        supported_protocols=route.supported_protocols,
        type=route.type
    )


def main():
    """Main execution"""
    AzureRMRoute()
    # x = CdnManagementClient()
    # x.routes.begin_update()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2016, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: nginx_status_info
short_description: Retrieve information on nginx status
description:
  - Gathers information from nginx from an URL having C(stub_status) enabled.
author: "René Moser (@resmo)"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  url:
    type: str
    description:
      - URL of the nginx status.
    required: true
  timeout:
    type: int
    description:
      - HTTP connection timeout in seconds.
    default: 10

notes:
  - See U(http://nginx.org/en/docs/http/ngx_http_stub_status_module.html) for more information.
"""

EXAMPLES = r"""
# Gather status info from nginx on localhost
- name: Get current http stats
  community.general.nginx_status_info:
    url: http://localhost/nginx_status
  register: result

# Gather status info from nginx on localhost with a custom timeout of 20 seconds
- name: Get current http stats
  community.general.nginx_status_info:
    url: http://localhost/nginx_status
    timeout: 20
  register: result
"""

RETURN = r"""
active_connections:
  description: Active connections.
  returned: success
  type: int
  sample: 2340
accepts:
  description: The total number of accepted client connections.
  returned: success
  type: int
  sample: 81769947
handled:
  description: The total number of handled connections. Generally, the parameter value is the same as accepts unless some
    resource limits have been reached.
  returned: success
  type: int
  sample: 81769947
requests:
  description: The total number of client requests.
  returned: success
  type: int
  sample: 144332345
reading:
  description: The current number of connections where nginx is reading the request header.
  returned: success
  type: int
  sample: 0
writing:
  description: The current number of connections where nginx is writing the response back to the client.
  returned: success
  type: int
  sample: 241
waiting:
  description: The current number of idle client connections waiting for a request.
  returned: success
  type: int
  sample: 2092
data:
  description: HTTP response as is.
  returned: success
  type: str
  sample: "Active connections: 2340 \nserver accepts handled requests\n 81769947 81769947 144332345 \nReading: 0 Writing: 241 Waiting: 2092 \n"
"""

import re

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.urls import fetch_url


class NginxStatusInfo:
    def __init__(self):
        self.url = module.params.get("url")
        self.timeout = module.params.get("timeout")

    def run(self):
        result = {
            "active_connections": None,
            "accepts": None,
            "handled": None,
            "requests": None,
            "reading": None,
            "writing": None,
            "waiting": None,
            "data": None,
        }
        (response, info) = fetch_url(module=module, url=self.url, force=True, timeout=self.timeout)
        if not response:
            module.fail_json(msg=f"No valid or no response from url {self.url} within {self.timeout} seconds (timeout)")

        data = to_text(response.read(), errors="surrogate_or_strict")
        if not data:
            return result

        result["data"] = data
        expr = (
            r"Active connections: ([0-9]+) \nserver accepts handled requests\n ([0-9]+) ([0-9]+) ([0-9]+) \n"
            r"Reading: ([0-9]+) Writing: ([0-9]+) Waiting: ([0-9]+)"
        )
        match = re.match(expr, data, re.S)
        if match:
            result["active_connections"] = int(match.group(1))
            result["accepts"] = int(match.group(2))
            result["handled"] = int(match.group(3))
            result["requests"] = int(match.group(4))
            result["reading"] = int(match.group(5))
            result["writing"] = int(match.group(6))
            result["waiting"] = int(match.group(7))
        return result


def main():
    global module
    module = AnsibleModule(
        argument_spec=dict(
            url=dict(type="str", required=True),
            timeout=dict(type="int", default=10),
        ),
        supports_check_mode=True,
    )

    nginx_status_info = NginxStatusInfo().run()
    module.exit_json(changed=False, **nginx_status_info)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from copy import deepcopy
from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Mapping
from typing import Optional
from typing import Sequence
from typing import Union

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from .botocore import normalize_boto3_result
from .tagging import boto3_tag_list_to_ansible_dict

BotoResource = Union[None, Mapping[str, Any]]
BotoResourceList = Union[None, Sequence[Mapping[str, Any]]]
AnsibleAWSResource = Union[None, Mapping[str, Any]]
AnsibleAWSResourceList = Union[None, Sequence[Mapping[str, Any]]]


def ansible_dict_to_boto3_filter_list(filters_dict):
    """Convert an Ansible dict of filters to list of dicts that boto3 can use
    Args:
        filters_dict (dict): Dict of AWS filters.
    Basic Usage:
        >>> filters = {'some-aws-id': 'i-01234567'}
        >>> ansible_dict_to_boto3_filter_list(filters)
        {
            'some-aws-id': 'i-01234567'
        }
    Returns:
        List: List of AWS filters and their values
        [
            {
                'Name': 'some-aws-id',
                'Values': [
                    'i-01234567',
                ]
            }
        ]
    """

    filters_list = []
    for k, v in filters_dict.items():
        filter_dict = {"Name": k}
        if isinstance(v, bool):
            filter_dict["Values"] = [str(v).lower()]
        elif isinstance(v, int):
            filter_dict["Values"] = [str(v)]
        elif isinstance(v, str):
            filter_dict["Values"] = [v]
        else:
            filter_dict["Values"] = v

        filters_list.append(filter_dict)

    return filters_list


def map_complex_type(complex_type, type_map):
    """
    Allows to cast elements within a dictionary to a specific type
    Example of usage:

    DEPLOYMENT_CONFIGURATION_TYPE_MAP = {
        'maximum_percent': 'int',
        'minimum_healthy_percent': 'int'
    }

    deployment_configuration = map_complex_type(module.params['deployment_configuration'],
                                                DEPLOYMENT_CONFIGURATION_TYPE_MAP)

    This ensures all keys within the root element are casted and valid integers
    """

    if complex_type is None:
        return
    new_type = type(complex_type)()
    if isinstance(complex_type, dict):
        for key in complex_type:
            if key in type_map:
                if isinstance(type_map[key], list):
                    new_type[key] = map_complex_type(complex_type[key], type_map[key][0])
                else:
                    new_type[key] = map_complex_type(complex_type[key], type_map[key])
            else:
                new_type[key] = complex_type[key]
    elif isinstance(complex_type, list):
        for i in range(len(complex_type)):
            new_type.append(map_complex_type(complex_type[i], type_map))
    elif type_map:
        return globals()["__builtins__"][type_map](complex_type)
    return new_type


def scrub_none_parameters(parameters, descend_into_lists=True):
    """
    Iterate over a dictionary removing any keys that have a None value

    Reference: https://github.com/ansible-collections/community.aws/issues/251
    Credit: https://medium.com/better-programming/how-to-remove-null-none-values-from-a-dictionary-in-python-1bedf1aab5e4

    :param descend_into_lists: whether or not to descend in to lists to continue to remove None values
    :param parameters: parameter dict
    :return: parameter dict with all keys = None removed
    """

    clean_parameters = {}

    for k, v in parameters.items():
        if isinstance(v, dict):
            clean_parameters[k] = scrub_none_parameters(v, descend_into_lists=descend_into_lists)
        elif descend_into_lists and isinstance(v, list):
            clean_parameters[k] = [
                scrub_none_parameters(vv, descend_into_lists=descend_into_lists) if isinstance(vv, dict) else vv
                for vv in v
            ]
        elif v is not None:
            clean_parameters[k] = v

    return clean_parameters


def _perform_nested_transforms(
    resource: Mapping[str, Any],
    nested_transforms: Optional[Mapping[str, Callable]],
) -> Mapping[str, Any]:
    if not nested_transforms:
        return resource

    for k, transform in nested_transforms.items():
        if k in resource:
            resource[k] = transform(resource[k])

    return resource


def boto3_resource_to_ansible_dict(
    resource: BotoResource,
    transform_tags: bool = True,
    force_tags: bool = True,
    normalize: bool = True,
    ignore_list: Optional[Sequence[str]] = None,
    nested_transforms: Optional[Mapping[str, Callable]] = None,
) -> AnsibleAWSResource:
    """
    Transforms boto3-style (CamelCase) resource to the ansible-style (snake_case).

    :param resource: a dictionary representing the resource
    :param transform_tags: whether or not to perform "tag list" to "dictionary" conversion on the "Tags" key
    :param normalize: whether resources should be passed through .botocore.normalize_boto3_result
    :param ignore_list: a list of keys, the contents of which should not be transformed
    :param nested_transforms: a mapping of keys to Callable, the Callable will only be passed the value for the key
                              in the resource dictionary
    :return: dictionary representing the transformed resource
    """
    if not resource:
        return resource
    ignore_list = ignore_list or []
    nested_transforms = nested_transforms or {}

    transformed_resource = deepcopy(resource)
    if normalize:
        transformed_resource = normalize_boto3_result(transformed_resource)
    transformed_resource = _perform_nested_transforms(transformed_resource, nested_transforms)
    ignore_list = [*ignore_list, *nested_transforms]
    camel_resource = camel_dict_to_snake_dict(transformed_resource, ignore_list=ignore_list)
    if transform_tags and "Tags" in resource:
        camel_resource["tags"] = boto3_tag_list_to_ansible_dict(resource["Tags"])
    if force_tags and "Tags" not in resource:
        camel_resource["tags"] = {}

    return camel_resource


def boto3_resource_list_to_ansible_dict(
    resource_list: BotoResourceList,
    transform_tags: bool = True,
    force_tags: bool = True,
    normalize: bool = True,
    ignore_list: Optional[Sequence[str]] = None,
    nested_transforms: Optional[Mapping[str, Callable]] = None,
) -> AnsibleAWSResourceList:
    """
    Transforms a list of boto3-style (CamelCase) resources to the ansible-style (snake_case).

    :param resource_list: a list of dictionaries representing the resources
    :param transform_tags: whether or not to perform "tag list" to "dictionary" conversion on the "Tags" key
    :param normalize: whether resources should be passed through .botocore.normalize_boto3_result()
    :param ignore_list: a list of keys, the contents of which should not be transformed
    :param nested_transforms: a mapping of keys to Callable, the Callable will only be passed the value for the key
                              in the resource dictionary
    :return: list of dictionaries representing the transformed resources
    """
    if not resource_list:
        return resource_list
    return [
        boto3_resource_to_ansible_dict(resource, transform_tags, force_tags, normalize, ignore_list, nested_transforms)
        for resource in resource_list
    ]


def sanitize_filters_to_boto3_filter_list(
    filters: Dict[str, Any], ignore_keys: Optional[List[str]] = None
) -> Dict[str, Any]:
    """
    Replace filter key underscores with dashes, for compatibility and transform ansible dict
    into boto3 filter list.

    :param filters: Ansible module params filters.
    :param ignore_keys: A list of keys to ignore.
    :return: Sanitized filters
    """
    sanitized_filters = deepcopy(filters)
    for k, v in filters.items():
        if ignore_keys and any((k.startswith(x) for x in ignore_keys)):
            continue
        if "_" in k:
            sanitized_filters[k.replace("_", "-")] = v
            del sanitized_filters[k]
    return ansible_dict_to_boto3_filter_list(sanitized_filters)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_server_pool_qualification_policy_info
short_description: Gather information about Server Pool Qualification Policies in Cisco Intersight
description:
  - Gather information about Server Pool Qualification Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all Server Pool Qualification Policies will be returned.
  - Server Pool Qualification Policies define conditions to qualify servers for resource pools based on hardware and configuration attributes.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the Server Pool Qualification Policy belongs to.
    type: str
  name:
    description:
      - The name of the Server Pool Qualification Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific Server Pool Qualification Policy by name
  cisco.intersight.intersight_server_pool_qualification_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-Qual-Policy-01"

- name: Fetch all Server Pool Qualification Policies in a specific Organization
  cisco.intersight.intersight_server_pool_qualification_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Production"

- name: Fetch all Server Pool Qualification Policies
  cisco.intersight.intersight_server_pool_qualification_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"

- name: Register Server Pool Qualification Policy info and display
  cisco.intersight.intersight_server_pool_qualification_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-Comprehensive-Policy"
  register: policy_info

- name: Display the policy qualifiers
  ansible.builtin.debug:
    var: policy_info.api_response[0].Qualifiers
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
    {
        "Name": "ServerPool-Qual-Policy-01",
        "ObjectType": "resourcepool.QualificationPolicy",
        "Organization": {
            "Moid": "675450ee69726530014753e2",
            "ObjectType": "organization.Organization"
        },
        "Qualifiers": [
            {
                "FabricInterConnectPids": ["UCS-FI-6454", "UCS-FI-64108"],
                "DomainNames": ["AC08-6454"],
                "ObjectType": "resource.DomainQualifier"
            },
            {
                "RackIdRange": [{"MinValue": 2, "MaxValue": 4}],
                "Pids": ["UCSC-C245-M8SX", "UCSC-C220-M8S"],
                "AssetTags": ["production"],
                "ObjectType": "resource.RackServerQualifier"
            },
            {
                "MemoryCapacityRange": {"MinValue": 64, "MaxValue": 512},
                "ObjectType": "resource.MemoryQualifier"
            },
            {
                "GpuEvaluationType": "ServerWithoutGpu",
                "ObjectType": "resource.GpuQualifier"
            },
            {
                "CpuCoresRange": {"MinValue": 16, "MaxValue": 64},
                "Vendor": "Intel(R) Corporation",
                "ObjectType": "resource.ProcessorQualifier"
            }
        ]
    },
    {
        "Name": "ServerPool-GPU-Policy",
        "ObjectType": "resourcepool.QualificationPolicy",
        "Description": "Policy for servers with NVIDIA GPUs",
        "Qualifiers": [
            {
                "GpuEvaluationType": "ServerWithGpu",
                "GpuCountRange": {"MinValue": 2, "MaxValue": 4},
                "Vendor": "NVIDIA",
                "ObjectType": "resource.GpuQualifier"
            }
        ]
    }
  ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to fetch info
    resource_path = '/resourcepool/QualificationPolicies'

    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import base64
import typing as t

from ansible import constants as C
from ansible.plugins.action import ActionBase
from ansible.utils.vars import merge_hash

from ansible_collections.community.docker.plugins.module_utils._scramble import (
    unscramble,
)


class ActionModule(ActionBase):
    # Set to True when transferring files to the remote
    TRANSFERS_FILES = False

    def run(
        self, tmp: str | None = None, task_vars: dict[str, t.Any] | None = None
    ) -> dict[str, t.Any]:
        self._supports_check_mode = True
        self._supports_async = True

        result = super().run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        # pylint: disable-next=no-member
        max_file_size_for_diff: int = C.MAX_FILE_SIZE_FOR_DIFF  # type: ignore
        self._task.args["_max_file_size_for_diff"] = max_file_size_for_diff

        result = merge_hash(
            result,
            self._execute_module(task_vars=task_vars, wrap_async=self._task.async_val),
        )

        if "diff" in result and result["diff"].get("scrambled_diff"):
            # Scrambling is not done for security, but to avoid no_log screwing up the diff
            diff = result["diff"]
            key = base64.b64decode(diff.pop("scrambled_diff"))
            for k in ("before", "after"):
                if k in diff:
                    diff[k] = unscramble(diff[k], key)

        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: ucs_server_maintenance
short_description: Creates Server Maintenance Policy on Cisco UCS Manager
description:
- Configures Server Maintenance Policy on Cisco UCS Manager.
extends_documentation_fragment: cisco.ucs.ucs
options:
  state:
    description:
    - If C(present), will verify Server Maintenance Policy is present and will create if needed.
    - If C(absent), will verify Server Maintenance Policy is absent and will delete if needed.
    choices: [present, absent]
    default: present
    type: str
  name:
    description:
    - The name assigned to the Server Maintenance Policy.
    - The Server Maintenance Policy name is case sensitive.
    - This name can be between 1 and 16 alphanumeric characters.
    - "You cannot use spaces or any special characters other than - (hyphen), \"_\" (underscore), : (colon), and . (period)."
    - You cannot change this name after the Server Maintenance Policy is created.
    required: yes
    type: str
  description:
    description:
    - A description of the Server Maintenance Package Policy.
    - Cisco recommends including information about where and when to use the policy.
    - Enter up to 256 characters.
    - "You can use any characters or spaces except the following:"
    - "` (accent mark), \ (backslash), ^ (carat), \" (double quote), = (equal sign), > (greater than), < (less than), or ' (single quote)."
    aliases: [ descr ]
    type: str
  trigger_config:
    description:
    - This option is used in combination with either User Ack (user-ack) or Timer Automatic (timer-automatic).
    - When the On Next Boot option is enabled, the host OS reboot, shutdown, or server reset also triggers the associated FSM to apply the changes.
    - Note that de-selecting the On Next Boot option disables the Maintenance Policy on the BMC.
    choices: [on-next-boot]
    type: str
  uptime_disr:
    description:
    - When a Server profile is associated with a Server, or when changes are made to an associated Server profile, you must reboot the Server.
    - The Reboot Policy field determines when the reboot occurs for Server associated with any Server profiles that include this maintenance policy.
    choices: [immediate, timer-automatic, user-ack]
    required: true
    type: str
requirements:
- ucsmsdk
author:
- Brett Johnson (@brettjohnson008)
'''

EXAMPLES = r'''
- name: Add Server Maintenance Policy
  cisco.ucs.ucs_server_maintenance:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: user-ack
    uptime_disr: user-ack
    trigger_config: on-next-boot
'''

RETURN = r'''
#
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ucs.plugins.module_utils.ucs import UCSModule, ucs_argument_spec


def main():
    argument_spec = ucs_argument_spec.copy()
    argument_spec.update(
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        trigger_config=dict(type='str', choices=['on-next-boot']),
        uptime_disr=dict(type='str', required=True, choices=['immediate', 'timer-automatic', 'user-ack']),
        state=dict(type='str', default='present', choices=['present', 'absent']),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    ucs = UCSModule(module)

    err = False

    # UCSModule creation above verifies ucsmsdk is present and exits on failure, so additional imports are done below.
    from ucsmsdk.mometa.lsmaint.LsmaintMaintPolicy import LsmaintMaintPolicy

    changed = False
    try:
        mo_exists = False
        props_match = False
        dn_base = 'org-root'
        dn = dn_base + '/maint-' + module.params['name']

        mo = ucs.login_handle.query_dn(dn)
        if mo:
            mo_exists = True

        if module.params['state'] == 'absent':
            # mo must exist but all properties do not have to match
            if mo_exists:
                if not module.check_mode:
                    ucs.login_handle.remove_mo(mo)
                    ucs.login_handle.commit()
                changed = True
        else:
            if mo_exists:
                # check top-level mo props
                kwargs = dict(name=module.params['name'])
                kwargs['descr'] = module.params['description']
                kwargs['trigger_config'] = module.params['trigger_config']
                kwargs['uptime_disr'] = module.params['uptime_disr']
                if mo.check_prop_match(**kwargs):
                    props_match = True

            if not props_match:
                if not module.check_mode:
                    # create if mo does not already exist
                    mo = LsmaintMaintPolicy(
                        parent_mo_or_dn=dn_base,
                        name=module.params['name'],
                        descr=module.params['description'],
                        trigger_config=module.params['trigger_config'],
                        uptime_disr=module.params['uptime_disr'],
                    )

                    ucs.login_handle.add_mo(mo, True)
                    ucs.login_handle.commit()
                changed = True

    except Exception as e:
        err = True
        ucs.result['msg'] = "setup error: %s " % str(e)

    ucs.result['changed'] = changed
    if err:
        module.fail_json(**ucs.result)
    module.exit_json(**ucs.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2016, Loic Blot <loic.blot@unix-experience.fr>
# Copyright (c) 2018, Ansible Project
# Sponsored by Infopro Digital. http://www.infopro-digital.com/
# Sponsored by E.T.A.I. http://www.etai.fr/
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: icinga2_feature

short_description: Manage Icinga2 feature
description:
  - This module can be used to enable or disable an Icinga2 feature.
author: "Loic Blot (@nerzhul)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - This is the feature name to enable or disable.
    required: true
  state:
    type: str
    description:
      - If set to V(present) and feature is disabled, then feature is enabled.
      - If set to V(present) and feature is already enabled, then nothing is changed.
      - If set to V(absent) and feature is enabled, then feature is disabled.
      - If set to V(absent) and feature is already disabled, then nothing is changed.
    choices: ["present", "absent"]
    default: present
"""

EXAMPLES = r"""
- name: Enable ido-pgsql feature
  community.general.icinga2_feature:
    name: ido-pgsql
    state: present

- name: Disable api feature
  community.general.icinga2_feature:
    name: api
    state: absent
"""

RETURN = r"""
#
"""

import re

from ansible.module_utils.basic import AnsibleModule


class Icinga2FeatureHelper:
    def __init__(self, module):
        self.module = module
        self._icinga2 = module.get_bin_path("icinga2", True)
        self.feature_name = self.module.params["name"]
        self.state = self.module.params["state"]

    def _exec(self, args):
        cmd = [self._icinga2, "feature"]
        rc, out, err = self.module.run_command(cmd + args, check_rc=True)
        return rc, out

    def manage(self):
        rc, out = self._exec(["list"])
        if rc != 0:
            self.module.fail_json(
                msg="Unable to list icinga2 features. Ensure icinga2 is installed and present in binary path."
            )

        # If feature is already in good state, just exit
        if (re.search(f"Disabled features:.* {self.feature_name}[ \n]", out) and self.state == "absent") or (
            re.search(f"Enabled features:.* {self.feature_name}[ \n]", out) and self.state == "present"
        ):
            self.module.exit_json(changed=False)

        if self.module.check_mode:
            self.module.exit_json(changed=True)

        feature_enable_str = "enable" if self.state == "present" else "disable"

        rc, out = self._exec([feature_enable_str, self.feature_name])

        change_applied = False
        if self.state == "present":
            if rc != 0:
                self.module.fail_json(
                    msg=f"Failed to {feature_enable_str} feature {self.feature_name}. icinga2 command returned {out}"
                )

            if re.search("already enabled", out) is None:
                change_applied = True
        else:
            if rc == 0:
                change_applied = True
            # RC is not 0 for this already disabled feature, handle it as no change applied
            elif re.search(f"Cannot disable feature '{self.feature_name}'. Target file .* does not exist", out):
                change_applied = False
            else:
                self.module.fail_json(msg=f"Failed to disable feature. Command returns {out}")

        self.module.exit_json(changed=change_applied)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            state=dict(type="str", choices=["present", "absent"], default="present"),
        ),
        supports_check_mode=True,
    )

    module.run_command_environ_update = dict(LANGUAGE="C", LC_ALL="C")
    Icinga2FeatureHelper(module).manage()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_evpn_ethernet
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_evpn_ethernet
short_description: Resource module to configure L2VPN EVPN Ethernet Segment.
description: This module manages the L2VPN EVPN Ethernet Segment attributes of Cisco IOS network devices.
version_added: 9.2.0
author:
  - Sagar Paul (@KB-perByte)
  - Jorgen Spange (@jorgenspange)
notes:
  - Tested against Cisco IOSXE Version 17.16.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
  - The module examples uses callback plugin (callback_result_format=yaml) to generate task
    output in yaml format.
options:
  config:
    description: A dictionary of L2VPN EVPN Ethernet Segment options
    type: list
    elements: dict
    suboptions:
      segment:
        description:
          - L2VPN EVPN Ethernet Segment, l2vpn evpn ethernet-segment 1
        type: str
        required: true
      default:
        description: Set a command to its defaults
        type: bool
      df_election:
        description: Designated forwarder election parameters
        type: dict
        suboptions:
          preempt_time:
            description: Preempt time before advertising routes
            type: int
          wait_time:
            description: Designated forwarder election wait time
            type: int
      identifier:
        description: Ethernet Segment Identifiers
        type: dict
        suboptions:
          identifier_type:
            description:
              - Type 0 (arbitrary 9-octet ESI value)
              - Type 3 (MAC-based ESI value)
            type: str
            choices:
              - '0'
              - '3'
          esi_value:
            description: system mac or 9-octet ESI value in hex
            type: str
      redundancy:
        description: Multi-homing redundancy parameters
        type: dict
        suboptions:
          all_active:
            description: Per-flow load-balancing between PEs on same Ethernet Segment
            type: bool
          single_active:
            description: Per-vlan load-balancing between PEs on same Ethernet Segment
            type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^l2vpn).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - purged
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | section ^l2vpn) executed on device. For state I(parsed) active
        connection to remote host is not required.
      - The state I(purged) negates virtual/logical interfaces that are specified in task
        from running-config.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
      - identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.01
        redundancy:
          single_active: true
        segment: '1'
      - df_election:
          preempt_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.02
        redundancy:
          single_active: true
        segment: '2'
      - identifier:
          identifier_type: '3'
          esi_value: 00.00.00.00.00.00.00.00.03
        redundancy:
          single_active: true
        segment: '3'
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.04
        redundancy:
          all_active: true
        segment: '4'
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.05
        redundancy:
          all_active: true
        segment: '5'
    state: merged

# Task Output
# -----------
#
# before:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
# commands:
# - l2vpn evpn ethernet-segment 5
# - identifier type 0 00.00.00.00.00.00.00.00.05
# - redundancy all-active
# - df-election wait-time 1
# after:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.05
#    redundancy:
#        all_active: true
#    segment: '5'

# After state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !
# l2vpn evpn ethernet-segment 5
#  identifier type 0 00.00.00.00.00.00.00.00.05
#  redundancy all-active
#  df-election wait-time 1
# !

# Using replaced

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.04
        redundancy:
          single_active: true
        segment: '4'
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.05
        redundancy:
          all_active: true
        segment: '5'
    state: replaced

# Task Output
# -----------
#
# before:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
# commands:
# - l2vpn evpn ethernet-segment 4
# - redundancy single-active
# - l2vpn evpn ethernet-segment 5
# - identifier type 0 00.00.00.00.00.00.00.00.05
# - redundancy all-active
# - df-election wait-time 1
# after:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        single_active: true
#    segment: '4'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.05
#    redundancy:
#        all_active: true
#    segment: '5'

# After state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy single-active
#  df-election wait-time 1
# !
# l2vpn evpn ethernet-segment 5
#  identifier type 0 00.00.00.00.00.00.00.00.05
#  redundancy all-active
#  df-election wait-time 1
# !

# Using overridden

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.04
        redundancy:
          single_active: true
        segment: '4'
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.05
        redundancy:
          all_active: true
        segment: '5'
    state: overridden

# After state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy single-active
#  df-election wait-time 1
# !
# l2vpn evpn ethernet-segment 5
#  identifier type 0 00.00.00.00.00.00.00.00.05
#  redundancy all-active
#  df-election wait-time 1
# !

# Using deleted

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
      - identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.01
        redundancy:
          single_active: true
        segment: '1'
      - df_election:
          preempt_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.02
        redundancy:
          single_active: true
        segment: '2'
    state: deleted

# Task Output
# -----------
#
# before:
#  - identifier:
#      identifier_type: '0'
#      esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#      single_active: true
#    segment: '1'
#  - df_election:
#      preempt_time: 1
#    identifier:
#      identifier_type: '0'
#      esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#      single_active: true
#    segment: '2'
#  - identifier:
#      identifier_type: '3'
#      esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#      single_active: true
#    segment: '3'
#  - df_election:
#      wait_time: 1
#    identifier:
#      identifier_type: '0'
#      esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#      all_active: true
#    segment: '4'
# commands:
# - l2vpn evpn ethernet-segment 1
# - no identifier type 0 00.00.00.00.00.00.00.00.01
# - no redundancy single-active
# - l2vpn evpn ethernet-segment 2
# - no identifier type 0 00.00.00.00.00.00.00.00.02
# - no redundancy single-active
# - no df-election wait-time 1
# after:
#  - segment: '1'
#  - segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.05
#    redundancy:
#        all_active: true
#    segment: '5'

# After state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
# !
# l2vpn evpn ethernet-segment 2
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !
# l2vpn evpn ethernet-segment 5
#  identifier type 0 00.00.00.00.00.00.00.00.05
#  redundancy all-active
#  df-election wait-time 1
# !

# Using purged

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
      - segment: '1'
      - segment: '2'
    state: purged

# Task Output
# -----------
#
# before:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
# commands:
# - no l2vpn evpn ethernet-segment 1
# - no l2vpn evpn ethernet-segment 2
# after:
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.05
#    redundancy:
#        all_active: true
#    segment: '5'

# After state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !
# l2vpn evpn ethernet-segment 5
#  identifier type 0 00.00.00.00.00.00.00.00.05
#  redundancy all-active
#  df-election wait-time 1
# !

# Using gathered

# Before state:
# -------------
#
# vios#sh running-config | section ^l2vpn evpn ethernet-segment
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Gather facts of evpn ethernet segment
  cisco.ios.ios_evpn_ethernet:
    config:
    state: gathered

# Task Output
# -----------
#
# gathered:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'

# Using rendered

- name: Render commands with provided configuration
  cisco.ios.ios_evpn_ethernet:
    config:
      - identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.01
        redundancy:
          single_active: true
        segment: '1'
      - df_election:
          preempt_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.02
        redundancy:
          single_active: true
        segment: '2'
      - identifier:
          identifier_type: '3'
          esi_value: 00.00.00.00.00.00.00.00.03
        redundancy:
          single_active: true
        segment: '3'
      - df_election:
          wait_time: 1
        identifier:
          identifier_type: '0'
          esi_value: 00.00.00.00.00.00.00.00.04
        redundancy:
          all_active: true
        segment: '4'
    state: rendered

# Task Output
# -----------
#
# rendered:
# - l2vpn evpn ethernet-segment 1
# - redundancy single-active
# - identifier type 0 00.00.00.00.00.00.00.00.01
# - l2vpn evpn ethernet-segment 2
# - df-election preempt-time 1
# - redundancy single-active
# - identifier type 0 00.00.00.00.00.00.00.00.02
# - l2vpn evpn ethernet-segment 3
# - redundancy single-active
# - identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
# - l2vpn evpn ethernet-segment 4
# - df-election wait-time 1
# - redundancy all-active
# - identifier type 0 00.00.00.00.00.00.00.00.04

# Using parsed

# File: parsed.cfg
# ----------------
#
# l2vpn evpn ethernet-segment 1
#  identifier type 0 00.00.00.00.00.00.00.00.01
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 2
#  identifier type 0 00.00.00.00.00.00.00.00.02
#  redundancy single-active
#  df-election preempt-time 1
# !
# l2vpn evpn ethernet-segment 3
#  identifier type 3 system-mac 00.00.00.00.00.00.00.00.03
#  redundancy single-active
# !
# l2vpn evpn ethernet-segment 4
#  identifier type 0 00.00.00.00.00.00.00.00.04
#  redundancy all-active
#  df-election wait-time 1
# !

- name: Parse the provided configuration
  cisco.ios.ios_evpn_ethernet:
    running_config: "{{ lookup('file', 'ios_ethernet_segment_parsed.cfg') }}"
    state: parsed

# Task Output
# -----------
#
# parsed:
#  - identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.01
#    redundancy:
#        single_active: true
#    segment: '1'
#  - df_election:
#        preempt_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.02
#    redundancy:
#        single_active: true
#    segment: '2'
#  - identifier:
#        identifier_type: '3'
#        esi_value: 00.00.00.00.00.00.00.00.03
#    redundancy:
#        single_active: true
#    segment: '3'
#  - df_election:
#        wait_time: 1
#    identifier:
#        identifier_type: '0'
#        esi_value: 00.00.00.00.00.00.00.00.04
#    redundancy:
#        all_active: true
#    segment: '4'
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - l2vpn evpn ethernet-segment 1
    - identifier type 0 00.00.00.00.00.00.00.00.01
    - redundancy single-active
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - l2vpn evpn ethernet-segment 1
    - identifier type 3 system-mac 0000.0000.0000.0001
    - redundancy all-active
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.evpn_ethernet.evpn_ethernet import (
    Evpn_ethernetArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.evpn_ethernet.evpn_ethernet import (
    Evpn_ethernet,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Evpn_ethernetArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Evpn_ethernet(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_connection
version_added: "3.19.0"
short_description: Create or Delete an Azure Machine Learning Connection
description:
    - Create or Delete an Azure Machine Learning Connection.
options:
    name:
        description:
            - Name of the Azure ML connection asset.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    resource_definition:
        description:
            - YAML definition containing the Azure ML Connection.
        type: str
        required: false
    populate_secrets:
        description:
            - For API key-based connections, try to populate the returned
              credentials with the actual secret value. Does nothing for
              non-API key-based connections.
        type: bool
        default: false
    state:
        description:
            - State of the Connection. Use C(present) to create.
              C(absent) to delete.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create GIT ML Connection
  azure.azcollection.azure_rm_ml_connection:
    name: MyGitConnection
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    resource_definition: |
      $schema: http://azureml/sdk-2-0/Connection.json
      name: test_ws_conn_git_pat
      type: git
      target: https://test-git-feed.com
      credentials:
        type: username_password
        username: dummy
        password: dummy
    tags:
      createdByToolkit: Ansible

- name: Create S3 ML Connection
  azure.azcollection.azure_rm_ml_connection:
    name: MyS3Connection
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    resource_definition: |
      $schema: http://azureml/sdk-2-0/Connection.json
      type: s3
      name: my_s3_connection
      target: <mybucket># add the s3 bucket details
      credentials:
          type: access_key
          access_key_id: XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX # add access key id
          secret_access_key: XxXxXxXXXXXXXxXxXxxXxxXXXXXXXXxXxxXXxXXXXXXXxxxXxXXxXXXXXxXXxXXXxXxXxxxXXxXXxXXXXXxXxxXX # add access key secret
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_connection:
    description:
        - Connection that was just created.
    returned: always
    type: dict
    sample: {
        "creation_context": {
            "created_at": "2026-05-11T20:07:52.118675+00:00",
            "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "created_by_type": "Application",
            "last_modified_at": "2026-05-11T20:07:52.118675+00:00",
            "last_modified_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "last_modified_by_type": "Application"
        },
        "credentials": {
            "type": "username_password"
        },
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-connection/providers/Microsoft.MachineLearningServices/workspaces/workspacexxxxxxxxxx/connections/connectionxxxxxxxxxx",
        "is_shared": true,
        "metadata": {},
        "name": "connectionxxxxxxxxxx",
        "tags": {},
        "target": "https://test-git-feed.com",
        "type": "git"
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_connection
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLConnection(MLClientCommon):

    PARAM_OVERRIDES = ['name', 'tags']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            populate_secrets=dict(
                type='bool',
                default=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_connection={}
        )

        required_if = [
            ('state', 'present', ['resource_definition'])
        ]
        super(AzureRMMLConnection, self).__init__(self.module_arg_spec,
                                                  supports_check_mode=True,
                                                  required_if=required_if)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        changed = False
        ml_connection_info = self.get(self.name)

        if self.state == 'present':
            if ml_connection_info:
                # Update
                self.module.warn('No Updates are supported at this time.')
                changed = False
                ml_connection_info = self.entity_to_dict(ml_connection_info)
            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_connection based on ansible args passed in.
                ml_connection = load_connection(resource_definition,
                                                params_override=params_override)
                changed = True
                if not self.check_mode:
                    response = self.client.connections.create_or_update(
                        ml_connection,
                        populate_secrets=self.populate_secrets
                    )
                    ml_connection_info = self.entity_to_dict(response)
        elif self.state == 'absent':
            # Delete
            if ml_connection_info:
                changed = True
                if not self.check_mode:
                    self.delete(self.name)
                ml_connection_info = self.get(self.name,
                                              as_dict=True)

        self.results['ml_connection'] = ml_connection_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides):
        params_override = []

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, as_dict=False):
        try:
            # Attempt to retrieve ml_connection based on name
            ml_connection_info = self.client.connections.get(
                name=name,
                populate_secrets=True)
            if as_dict:
                ml_connection_info = self.entity_to_dict(ml_connection_info)
        except ResourceNotFoundError:
            ml_connection_info = None
        except Exception as e:  # pylint: disable=broad-exception-caught
            ml_connection_info = None
        return ml_connection_info

    def delete(self, name=None):
        try:
            response = self.client.connections.delete(name=name)
            return response
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error delete the data instance: {0}".format(str(e)))


def main():
    AzureRMMLConnection()


if __name__ == '__main__':
    main()
